// textkbd.cpp
// VERSION 2
// Glenn G. Chappell
// 8 Sep 2011
//
// For CS 381 Fall 2011
// GLUT Text & Keyboard Demo

#include <sstream>
using std::ostringstream;
#include <string>
using std::string;
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <cstdlib>       // Do this before GL/GLUT includes
using std::exit;
#ifndef __APPLE__
# include <GL/glut.h>    // GLUT stuff, includes OpenGL headers as well
#else
# include <GLUT/glut.h>  // Apple puts glut.h in a different place
#endif


// Global variables
// Keyboard
const int ESCKEY = 27;         // ASCII value of Escape

// Window/viewport
const int startwinsize = 400;  // Start window width & height (pixels)

// Scene
bool showsquare;               // True if big square displayed
double redlevel;               // Red component of square color


// class BitmapPrinter
// For drawing text using GLUT bitmap text facility.
// Usage:
//     BitmapPrinter p(-0.9, 0.9, 0.1); // Start text x, y, line height
//     p.print("Hello");                // 1st line, start @ (-0.9, 0.9)
//     p.print("there");                // 2nd line, start @ (-0.9, 0.8)
class BitmapPrinter {

// ***** BitmapPrinter: public functions *****
public:

    // Ctor (0, 1, 2, or 3 params)
    // Set cursx, cursy, lineht to the given values.
    BitmapPrinter(double theCursx = 0.,
                  double theCursy = 0.,
                  double theLineht = 0.1)
    { setup(theCursx, theCursy, theLineht); }

    // Compiler-generated copy ctor, copy =, dctor used

    // setup
    // Set cursx, cursy, lineht to the given values.
    void setup(double theCursx,
               double theCursy,
               double theLineht = 0.1)
    { cursx = theCursx; cursy = theCursy; lineht = theLineht; }

    // print
    // Draw the given string, using glutBitmapCharacter, with GLUT's
    // 9x15 font, at cursx, cursy, and then reduce cursy by lineht
    // (i.e., move to the next line).
    //
    // The model/view transformation should probably be the identity
    // (or just translations) when calling this function.
    void print(const std::string & msg)
    {
        glRasterPos2d(cursx, cursy);
        for (std::string::const_iterator ii = msg.begin();
             ii != msg.end();
             ++ii)
        {
            glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *ii);
        }
        cursy -= lineht;
    }

// ***** BitmapPrinter: data members *****
private:

    double cursx, cursy;  // Initial raster pos for text line: x, y
    double lineht;        // How much to reduce cursy each line

};  // End class BitmapPrinter


// myDisplay
// The GLUT display function
void myDisplay()
{
    glClear(GL_COLOR_BUFFER_BIT);

    // Draw objects
    if (showsquare)
    {
        glColor3d(redlevel, 0.3, 0.3);
        glBegin(GL_QUADS);
            glVertex3d(-0.7,-0.7,0.);
            glVertex3d( 0.7,-0.7,0.);
            glVertex3d( 0.7, 0.7,0.);
            glVertex3d(-0.7, 0.7,0.);
        glEnd();
    }
    glColor3d(0.3, 0.9, 0.3);
    glBegin(GL_QUADS);
        glVertex3d(-0.85,0.5 ,0.);
        glVertex3d(-0.5 ,0.5 ,0.);
        glVertex3d(-0.5 ,0.85,0.);
        glVertex3d(-0.85,0.85,0.);
    glEnd();
    glColor3d(0.3, 0.3, 0.9);
    glBegin(GL_QUADS);
        glVertex3d(0.85,-0.5 ,0.);
        glVertex3d(0.5 ,-0.5 ,0.);
        glVertex3d(0.5 ,-0.85,0.);
        glVertex3d(0.85,-0.85,0.);
    glEnd();

    // Draw documentation
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);  // Set up simple ortho projection
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(-1., 1., -1., 1.);
    glColor3d(0., 0., 0.);        // Black text
    BitmapPrinter p(-0.9, 0.9, 0.1);
    p.print("This is bitmap text");
    ostringstream ostr;
    ostr << (2+3)*14;
    p.print("A number converted to a string: " + ostr.str());
    p.print("Space    Toggle big square");
    p.print("<- ->    Change red in big square color");
    p.print("Esc      Quit");
    glPopMatrix();                // Restore prev projection
    glMatrixMode(GL_MODELVIEW);

    glFlush();
}


// myIdle
// The GLUT idle function
void myIdle()
{
    // Print OpenGL errors, if there are any (for debugging)
    if (GLenum err = glGetError())
    {
        cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
    }
}


// myKeyboard
// The GLUT keyboard function
void myKeyboard(unsigned char key, int x, int y)
{
    switch (key)
    {
    case ESCKEY:  // Esc: quit
        exit(0);
        break;
    case ' ':     // Space: toggle square
        showsquare = !showsquare;
        glutPostRedisplay();
        break;
    }
}


// mySpecial
// The GLUT special function
void mySpecial(int key, int x, int y)
{
    switch (key)
    {
    case GLUT_KEY_RIGHT:  // -> increase red
        redlevel += 0.1;
        if (redlevel > 1.)
            redlevel = 1.;
        glutPostRedisplay();
        break;
    case GLUT_KEY_LEFT:   // <- decrease red
        redlevel -= 0.1;
        if (redlevel < 0.)
            redlevel = 0.;
        glutPostRedisplay();
        break;
    }
}


// myReshape
// The GLUT reshape function
void myReshape(int w, int h)
{
    // Set viewport
    glViewport(0, 0, w, h);

    // Set up projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-1., 1., -1., 1.);

    glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
    // Scene
    showsquare = true;
    redlevel = 0.7;

    // GL stuff
    glClearColor(0.7, 0.7, 0.7, 0.0);
}


int main(int argc, char ** argv)
{
    // Initialize OpenGL/GLUT
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);

    // Make a window
    glutInitWindowSize(startwinsize, startwinsize);
    glutInitWindowPosition(50, 50);
    glutCreateWindow("CS 381 - GLUT Text & Keyboard Demo");

    // Initialize GL states & register GLUT callbacks
    init();
    glutDisplayFunc(myDisplay);
    glutIdleFunc(myIdle);
    glutKeyboardFunc(myKeyboard);
    glutSpecialFunc(mySpecial);
    glutReshapeFunc(myReshape);

    // Do something
    glutMainLoop();

    return 0;
}

